/****************************************************************************
** Demonstration of drawing a line                                         **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include "dos.h"
#include "stdlib.h"

unsigned char *Screen = (unsigned char *)0xA0000000L;
unsigned char *Buffer;

//This sets the display to VGA 320x200 in 256 colours
void VGAScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x13;
  int86(0x10, &r, &r);
}

//This resets the display to text mode
void TextScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x3;
  int86(0x10, &r, &r);
}

//Sets a pixel in the buffer to a specific colour
void PutPixel (int x, int y, char c)
{
  Buffer [(y << 8) + (y << 6) + x] = c;
}

//Draws a line between two points
void DrawLine (int x1, int y1, int x2, int y2, unsigned char c)
{
  int   i, deltax, deltay, numpixels,
        d, dinc1, dinc2,
        x, xinc1, xinc2,
        y, yinc1, yinc2;

  //Calculate deltax and deltay for initialisation
  deltax = abs (x2 - x1);
  deltay = abs (y2 - y1);

  //Initialize all vars based on which is the independent variable
  if (deltax >= deltay) {
    //x is independent variable
    numpixels = deltax + 1;
    d = (deltay << 1) - deltax;
    dinc1 = deltay << 1;
    dinc2 = (deltay - deltax) << 1;
    xinc1 = xinc2 = yinc2 = 1;
    yinc1 = 0;
  } else {
    //y is independent variable
    numpixels = deltay + 1;
    d = (deltax << 1) - deltay;
    dinc1 = deltax << 1;
    dinc2 = (deltax - deltay) << 1;
    xinc1 = 0;
    xinc2 = yinc1 = yinc2 = 1;
  }

  //Make sure x and y move in the right directions
  if (x1 > x2) {
    xinc1 = -xinc1;
    xinc2 = -xinc2;
  }
  if (y1 > y2) {
    yinc1 = - yinc1;
    yinc2 = - yinc2;
  }

  //Start drawing at <x1, y1>
  x = x1;
  y = y1;

  //Draw the pixels
  for (i = 0; i < numpixels; i++) {
    PutPixel (x, y, c);
    if (d < 0) {
      d += dinc1;
      x += xinc1;
      y += yinc1;
    } else {
      d += dinc2;
      x += xinc2;
      y += yinc2;
    }
  }
}

void main ()
{
  //Random coordinates and colour
  int x1, y1, x2, y2, colour;

  //Switch to graphics mode
  VGAScreen ();

  //I'm drawing to the screen instead of a buffer
  Buffer = Screen;

  //Keep drawing lines until a keypress
  do {
    x1 = random (320);
    y1 = random (200);
    x2 = random (320);
    y2 = random (200);
    colour = random (256);
    DrawLine (x1, y1, x2, y2, colour);
  } while (!kbhit ());

  getch ();
  TextScreen ();
}